% --------Synthetic experiment-----
% ---------------------------------
clear;
clc;

fprintf('\n \n ------ Running Synthetic experiment [Table 1] ------\n ');


%% Parameters setup

num_runs = 1;
num_methods = 4;

dataset = 'synthetic'; % synthetic

easy = 0;          % for synthetic dataset:
% 1 - image with a good visual contrast
% 0 - image with a low visual contrast

ekm_map = 0;       % 1 - use EKM based mapping of original image
% 0 - use original image

SNR = 10;          % per-channel SNR for AWGN

basis_learning = 2; % 1- kmeans
% 2 - FVS

kernel = 'gaussian';
var1 = 1e-1;        % variance of Gaussian kernel
var2 = 1e-1;

method = 1;       % 1 - LRR SSC for problem with noise
% 2 - LRRS SSC for problem with noise (ADMM algorithm)

tau = 0.1;        % threshold for closed form solution of the LRR SC problem with noise

% constants for ADMM algorithm for LRRS SSC problem with noise

beta_1 = 1;       % penalty for LRR (nuclear norm) constraint
beta_2 = 1;       % penalty for sparseness (L1 norm) constraint

mu_2 = 0.1;       % penalty for off-diagonality constraint and learning gain for LAM_2
mu_3 = 0.1;       % penalty for J=C1 equality constraint and learning gain for LAM_3

%set variance according to different signal-to-noise ratio (FIXME)
if SNR>=18 && SNR<=28
    var1 = 0.01;
elseif SNR<=17
    var1 = 0.1;
end

%% load dataset
switch(dataset)
    case('synthetic')
        d = 1; %dimensionality of subspace
        L = 5; %number of clusters (clusters=subspaces - 1D subspaces)
        
        S(:,:,1)=zeros(100,100);
        S(10:30,10:30,1)=1;
        S(:,:,2)=zeros(100,100);
        S(70:90,10:30,2)=1;
        S(:,:,3)=zeros(100,100);
        S(10:30,70:90,3)=1;
        S(:,:,4)=zeros(100,100);
        S(70:90,70:90,4)=1;
        S(:,:,5) = ones(100,100);
        S(:,:,5) = S(:,:,5).*(abs(1-S(:,:,1)));
        S(:,:,5) = S(:,:,5).*(abs(1-S(:,:,2)));
        S(:,:,5) = S(:,:,5).*(abs(1-S(:,:,3)));
        S(:,:,5) = S(:,:,5).*(abs(1-S(:,:,4)));
        tmp=S;
        clear S
        [I1 I2 I3] = size(tmp);
        for i3=1:I3
            S(i3,:)=reshape(tmp(:,:,i3),1,I1*I2);
        end
        clear tmp
        S = max(0,S);
        
        if easy == 1 % easy problem
            a1 = [1.0 0 0]';
            a2 = [0 1 0]';
            a3 = [0 0 1]';
            a4 = [0.5 0.5 0]';
            a5 = [0.5 0 0.5]';
            A = [a1 a2 a3 a4 a5];
        elseif easy == 0 % hard problem
            a1 = [1.0 2.5574 2.7183]';
            a2 = [1.0 2.0 1.7854]';
            a3 = [0.7854 2.0 2.0]';
            a4 = [1.0 1.7854 2.0]';
            a5 = [1.0 2.7183 2.5574]';
            A = [a1 a2 a3 a4 a5];
        end
        
        normA = normc(A);
        mutual_coherence = max(max(abs(triu((normA'*normA),1))))
        
        X = cell(num_runs,1);
        Gt_clusters = cell(num_runs,1);
        
        for run=1 : num_runs
            X_tmp=A*S;  % vectorized synthetic low-contrast RGB image
            
            for n=1:3
                xv=var(X_tmp(n,:));
                nv = xv*power(10,-SNR/10);
                X_tmp(n,:) = max(0,X_tmp(n,:)+sqrt(nv)*randn(1,100*100));
            end
            X_tmp = max(0,X_tmp);
            
            xmax = max(max(X_tmp));
            X_tmp = X_tmp/xmax;
            
            % for color coded ground truth
            [Gt_clusters_tmp, ~] = find(S); % indexes of rows with ones are cluster indexes
            
            X{run} = X_tmp;
            Gt_clusters{run} = Gt_clusters_tmp;
            % color coded ground truth image
            %b1 = [1 0 0]';
            %b2 = [0 1 0]';
            %b3 = [0 0 1]';
            %b4 = [0.6 0.3 0.4]';
            %b5 = [0.3 0.4 0.6]';
            %B = [b1 b2 b3 b4 b5];
            %Gt = B*S;
            %for i=1:3
            %   tmp(:,:,i)=reshape(Gt(i,:),I1,I2);
            %end
            %imagesc(tmp)
            %title('Color coded ground truth')
        end
        
        
end

%%
elapsed_time = zeros(num_methods, num_runs); %record elapsed time of each algorithm
accuracy = zeros(num_methods, num_runs); %record accuracy of each algorithm
optimal_tau = zeros(num_runs); %optimal tau for LRR
optimal_tau2 = zeros(num_runs); %FIXME REMOVEME optimal tau for LRR

for run=1:num_runs
    
    X_original = X{run};
    cur_Gt = Gt_clusters{run};
    
    D_ekm = d*L;
    tic;
    [X_ekm, V_ekm] = project_data(X_original, D_ekm, basis_learning, kernel, var1, var2);
    projection_time = toc;
    
    
    %% Subspace clustering with finding maximum value
    
    fprintf('Running subspace clustering with finding maximum value...\n');
    algorithm_idx = 1;
    
    tic;  
    idx = FELRSC(X_original, D_ekm, kernel, var1, var2);
       
    elapsed_time(algorithm_idx,run) = toc;
    accuracy(algorithm_idx,run) = 1-computeCE(idx,cur_Gt)
    
    %% Greedy subspace clustering
    fprintf('Running greedy subspace clustering...\n');
    algorithm_idx = 2;
    
    if ekm_map == 0
        cur_X = X_original;
    elseif ekm_map == 1
        cur_X = X_ekm;
    end
    
    tic;
    
    %Step 1. Constructing neighborhood matrix W
    k = 1;  % number of neighbours: subspace dimension-1
    
    %set parameters as recommended by authors
    if strcmp(dataset, 'synthetic')
        %number of neighbours depends on SNR
        if SNR==10 && ekm_map==0
            k = 2;
        elseif SNR==10 && ekm_map==1
            k = 3;
        elseif SNR==20
            k = 2;
        elseif SNR>=30
            k = 1;
        end
        k_max = k;
    elseif strcmp(dataset,'yaleb')
        k = 20;
        k_max = 20;
    elseif strcmp(dataset,'hopkins')
        k = 5;
        k_max = 5;
    end
    
    [Z,U] = NSN(cur_X, k, k_max);  % U is the neighborhood matrix (FIXME zar nije Z)
    
    %Step 2. Greedy subspace recovery
    N = size(cur_X,2);
    epsilon = 1e-4;
    D = cell(1,L); I = 1:N; l = 1;
    for l=1:L
        npoints = zeros(1,numel(I)); X1 = cur_X(:,I);
        for i=1:numel(I)
            npoints(i) = nnz(sum((X1-U{I(i)}*(U{I(i)}'*X1)).^2,1) < epsilon);
        end
        [~,maxindex] = max(npoints);
        
        D{l} = U{I(maxindex)};
        
        J = find(sum((X1-D{l}*(D{l}'*X1)).^2,1) < epsilon);
        I = setdiff(I,I(J));
    end
    A = LabelPoints(cur_X,D);
    
    %A = SpectralClusteringL(Z+Z',L); %instead of greedy subspace
    %recovery, spectral clustering step can be used
    
    elapsed_time(algorithm_idx,run) = toc;
    if ekm_map==1
        elapsed_time(algorithm_idx,run) = elapsed_time(algorithm_idx,run)+projection_time;
    end
    accuracy(algorithm_idx,run) = 1-computeCE(A,cur_Gt)
    
    %% Landmark points (learned basis) approach to computationally efficient subspace clustering
    %addpath('LSC');
    fprintf('Running landmark points (learned basis) approach...\n');
    algorithm_idx = 3;
    
    opts.r = d;
    opts.p = 1000;
    %opts.p = D_ekm;
    tic;
    idx = LSC(X_original',L,opts);
    elapsed_time(algorithm_idx,run) = toc;
    
    accuracy(algorithm_idx,run) = 1-computeCE(idx,cur_Gt)
    
    %% LRR solution for problem with noise
    
    fprintf('Running LRR solution for problem with noise...\n');
    algorithm_idx = 4;
    
    tau = [0.01 0.1,0.2,0.5]; % we need to find optimal tau
    
    accuracy_by_tau = zeros(length(tau),1);
    time_by_tau = zeros(length(tau),1);
    
    for t = 1:length(tau)
        t
        tic;
        
        if ekm_map == 0
            [idx] = lrr_SC(X_original, size(X_original,1), L, tau(t));
        elseif ekm_map == 1
            [idx] = lrr_SC(X_ekm, D_ekm, L, tau(t));
        end
        
        time_by_tau(t) = toc;
        if ekm_map==1
            time_by_tau(t) = time_by_tau(t)+projection_time;
        end
        accuracy_by_tau(t) = 1-computeCE(idx, cur_Gt);
        
        [maxAccuracy, maxIdx] = max(accuracy_by_tau);
        accuracy(algorithm_idx,run) = maxAccuracy;
        elapsed_time(algorithm_idx,run) = time_by_tau(maxIdx);
        optimal_tau(run) = tau(maxIdx);
        
    end
    
    
    fprintf('Accuracy: \n FELRSC %f \n NSN-GSR %f \n LSC %f \n LRR %f\n ', accuracy(1), accuracy(2), accuracy(3), accuracy(4));
    
end